@@ -1,2 +1,2 @@
 //! A module to re-export types that are visible through the ic-agent API.
-pub use ic_types::{Principal, PrincipalError};
+pub use crate::ic_types::{Principal, PrincipalError};
